﻿#include "installingframe.h"
#include <QGridLayout>
#include <QEvent>
#include <QFile>
#include <QDebug>
#include <QTextStream>
#include <QApplication>
#include <sys/stat.h>
#include <QThread>
#include "../PluginService/ui_unit/xrandrobject.h"
#define PROGRESS_RATIO 0.8
namespace KInstaller {
using namespace KServer;
InstallingFrame::InstallingFrame(QWidget *parent) : QWidget(parent),
    tmpvalue(0)
{
    initUI();
    addStyleSheet();
    progressBarCtrl = new ShowProgressBar(this);

    initAllConnect();
    translateStr();
    //恢复鼠标正常状态
    QApplication::restoreOverrideCursor();
    slide->setFocus();
}

InstallingFrame::~InstallingFrame()
{
    if(w != nullptr) {
        w->close();
    }
}

void InstallingFrame::initUI()
{
    QGridLayout* gridLayout = new QGridLayout;
    this->setLayout(gridLayout);
    gridLayout->setColumnStretch(0, 1);
    gridLayout->setColumnStretch(1, 5);
    gridLayout->setColumnStretch(2, 1);
    // gridLayout->setSpacing(20);
//    gridLayout->setContentsMargins(20, 0, 20, 20);

//    m_mainTitle = new QLabel(this);
//    m_mainTitle->setObjectName("mainTitle");
//    gridLayout->addItem(new QSpacerItem(10, 40, QSizePolicy::Expanding, QSizePolicy::Minimum), 0, 1, 1, 1);
//    gridLayout->addWidget(m_mainTitle, 1, 1, 1, 1, Qt::AlignCenter | Qt::AlignTop);
//    gridLayout->addItem(new QSpacerItem(10, 40, QSizePolicy::Expanding, QSizePolicy::Minimum), 2, 1, 1, 1);

    slide = new SlideShow(this);
    slide->setFocus();
//    slide->setMaximumSize(QSize(500, 300));
//    slide->setMinimumSize(QSize(500, 300));
    gridLayout->addWidget(slide, 0, 1, 1, 1, Qt::AlignCenter);
//    gridLayout->setRowStretch(0, 7);
//    gridLayout->setRowStretch(1, 1);
//    gridLayout->setRowStretch(3, 1);
    littleTitle = new QLabel;
    littleTitle->setObjectName("littleTitle");
    gridLayout->addItem(new QSpacerItem(10, 40, QSizePolicy::Expanding, QSizePolicy::Minimum), 1, 1, 1, 1);
    gridLayout->addWidget(littleTitle, 2, 1, 1, 1, Qt::AlignCenter);
    gridLayout->addItem(new QSpacerItem(10, 20, QSizePolicy::Expanding, QSizePolicy::Minimum), 3, 1, 1, 1);
    m_installProgressBar = new QProgressBar(this);
    m_installProgressBar->setTextVisible(false);
    m_installProgressBar->setValue(0);
    m_installProgressBar->setMaximumHeight(10);
    m_installProgressBar->setMinimumHeight(10);
    QRect rect = KServer::GetScreenRect();
    m_installProgressBar->setFixedWidth(rect.width() * PROGRESS_RATIO);
    m_installProgressBar->setObjectName(QStringLiteral("installProgressBar"));

    gridLayout->addWidget(m_installProgressBar, 4, 1, 1, 1);
//    gridLayout->addItem(new QSpacerItem(10, 40, QSizePolicy::Expanding, QSizePolicy::Minimum), 5, 1, 1, 1);

    w = new ArrowWidget(false, this);
//    w->setParent(this);
    w->setObjectName("ArrowWidget");
    w->setDerection(ArrowWidget::Derection::down);
    w->setWindowFlags(w->windowFlags() | Qt::Dialog);
}

void InstallingFrame::showToopTipValue(int value)
{

//    w->setText(QString("%1%").arg(value));//        w->setCenterWidget(textLabel);
    w->setText(QString("%1%").arg(value));
    QPoint pos = m_installProgressBar->mapToGlobal(QPoint());
    qDebug() << "pos:" << pos;
    w->move(pos.x() + m_installProgressBar->width() * value / 100 - w->width() / 2, pos.y() - w->height() - 2);
    qDebug() << "w:pos:" << w->pos();
    w->show();
}


void InstallingFrame::changeEvent(QEvent *event)
{
    if (event->type() == QEvent::LanguageChange) {
        translateStr();
    } else {
        QWidget::changeEvent(event);
    }
}

void InstallingFrame::addStyleSheet()
{
    QFile file(":/data/qss/InstallingFrame.css");
    file.open(QFile::ReadOnly);
    QTextStream filetext(&file);
    QString stylesheet = filetext.readAll();
    file.close();

    this->setStyleSheet(stylesheet);
}

void InstallingFrame::translateStr()
{
//    m_mainTitle->setText((tr("Installing")));//安装进行中
    littleTitle->setText(tr("The system is being installed, please do not turn off the computer"));//系统正在安装，请不要关闭计算机
}

void InstallingFrame::initAllConnect()
{
    connect(progressBarCtrl, &ShowProgressBar::signelProgressValue, this, &InstallingFrame::updateProgressValue);
    connect(m_installProgressBar, &QProgressBar::valueChanged, this, &InstallingFrame::showToopTipValue);
//    connect(this, &InstallingFrame::destroyed, w, &ArrowWidget::close);

}
void InstallingFrame::clickNextButton()
{

}

void InstallingFrame::findFile()
{
//    qDebug() << "findFile0" ;
//    QFile file("/tmp/z-finished-install");
//    if(file.exists()) {
//        qDebug() << "findFile1";
//        m_installProgressBar->setValue(100);
//        emit signalFinished();
//        timer->stop();
//    }
    m_installProgressBar->setValue(100);
    emit signalFinished();
}

void InstallingFrame::updateProgressValue(int value)
{
    m_installProgressBar->setValue(value);
    if (value == 90) {
        qDebug() << "updateProgressValue" << value;
//        timer->start();
        m_installProgressBar->setValue(96);
    }
}

void InstallingFrame::closeW()
{
    progressBarCtrl->timer->stop();
//    w->hide();
    w->close();

}



}
